@@ -1,5 +1,5 @@
 /* tc-dlx.c -- Assemble for the DLX
-   Copyright (C) 2002-2024 Free Software Foundation, Inc.
+   Copyright (C) 2002-2025 Free Software Foundation, Inc.
 
    This file is part of GAS, the GNU Assembler.
 
@@ -217,7 +217,7 @@ s_proc (int end_p)
 
       current_name = current_label = NULL;
       SKIP_WHITESPACE ();
-      while (!is_end_of_line[(unsigned char) *input_line_pointer])
+      while (!is_end_of_stmt (*input_line_pointer))
         input_line_pointer++;
     }
   else
@@ -499,12 +499,12 @@ dlx_parse_storeop (char * str)
 	pb = comma;
 
       /* Duplicate the first register.  */
-      for (i = comma + 1; (str[i] == ' ' || str[i] == '\t'); i++)
+      for (i = comma + 1; is_whitespace (str[i]); i++)
 	;
 
       for (m2 = 0; (m2 < 7 && str[i] != '\0'); i++, m2++)
 	{
-	  if (str[i] != ' ' && str[i] != '\t')
+	  if (!is_whitespace (str[i]))
 	    rd[m2] = str[i];
 	  else
 	    goto badoperand_store;
@@ -672,12 +672,12 @@ machine_ip (char *str)
     case '\0':
       break;
 
-      /* FIXME-SOMEDAY more whitespace.  */
-    case ' ':
-      *s++ = '\0';
-      break;
-
     default:
+      if (is_whitespace (*s))
+	{
+	  *s++ = '\0';
+	  break;
+	}
       as_bad (_("Unknown opcode: `%s'"), str);
       return;
     }
@@ -1169,9 +1169,11 @@ tc_gen_reloc (asection *section ATTRIBUTE_UNUSED,
 {
   arelent * reloc;
 
-  reloc = XNEW (arelent);
-  reloc->howto = bfd_reloc_type_lookup (stdoutput, fixP->fx_r_type);
+  reloc = notes_alloc (sizeof (arelent));
+  reloc->sym_ptr_ptr = notes_alloc (sizeof (asymbol *));
+  *reloc->sym_ptr_ptr = symbol_get_bfdsym (fixP->fx_addsy);
 
+  reloc->howto = bfd_reloc_type_lookup (stdoutput, fixP->fx_r_type);
   if (reloc->howto == NULL)
     {
       as_bad_where (fixP->fx_file, fixP->fx_line,
@@ -1183,8 +1185,6 @@ tc_gen_reloc (asection *section ATTRIBUTE_UNUSED,
 
   gas_assert (!fixP->fx_pcrel == !reloc->howto->pc_relative);
 
-  reloc->sym_ptr_ptr = XNEW (asymbol *);
-  *reloc->sym_ptr_ptr = symbol_get_bfdsym (fixP->fx_addsy);
   reloc->address = fixP->fx_frag->fr_address + fixP->fx_where;
 
   if (fixP->fx_r_type == BFD_RELOC_VTABLE_ENTRY)
