@@ -1,5 +1,5 @@
 /* tc-epiphany.c -- Assembler for the Adapteva EPIPHANY
-   Copyright (C) 2009-2024 Free Software Foundation, Inc.
+   Copyright (C) 2009-2025 Free Software Foundation, Inc.
    Contributed by Embecosm on behalf of Adapteva, Inc.
 
    This file is part of GAS, the GNU Assembler.
@@ -314,7 +314,7 @@ static const unsigned char nop_pattern[] = { 0xa2, 0x01 };
 void
 epiphany_handle_align (fragS *fragp)
 {
-  int bytes, fix;
+  int bytes;
   char *p;
 
   if (fragp->fr_type != rs_align_code)
@@ -322,23 +322,15 @@ epiphany_handle_align (fragS *fragp)
 
   bytes = fragp->fr_next->fr_address - fragp->fr_address - fragp->fr_fix;
   p = fragp->fr_literal + fragp->fr_fix;
-  fix = 0;
 
   if (bytes & 1)
     {
-      fix = 1;
       *p++ = 0;
-      bytes--;
+      fragp->fr_fix++;
     }
 
-  if (bytes & 2)
-    {
-      memcpy (p, nop_pattern, 2);
-      p += 2;
-      bytes -= 2;
-      fix += 2;
-    }
-  fragp->fr_fix += fix;
+  memcpy (p, nop_pattern, 2);
+  fragp->fr_var = 2;
 }
 
 /* Read a comma separated incrementing list of register names
@@ -353,7 +345,7 @@ parse_reglist (const char * s, int * mask)
     {
       long value;
 
-      while (*s == ' ')
+      while (is_whitespace (*s))
 	++s;
 
       /* Parse a list with "," or "}" as limiters.  */
@@ -371,7 +363,7 @@ parse_reglist (const char * s, int * mask)
 	return _("register is out of order");
       *mask |= regmask;
 
-      while (*s==' ')
+      while (is_whitespace (*s))
 	++s;
 
       if (*s == '}')
