@@ -1,5 +1,5 @@
 /* tc-moxie.c -- Assemble code for moxie
-   Copyright (C) 2009-2020 Free Software Foundation, Inc.
+   Copyright (C) 2009-2025 Free Software Foundation, Inc.
 
    This file is part of GAS, the GNU Assembler.
 
@@ -32,7 +32,7 @@ const char line_separator_chars[] = ";";
 const char line_comment_chars[]   = "#";
 
 static int pending_reloc;
-static struct hash_control *opcode_hash_control;
+static htab_t opcode_hash_control;
 
 const pseudo_typeS md_pseudo_table[] =
 {
@@ -62,19 +62,17 @@ md_begin (void)
 {
   int count;
   const moxie_opc_info_t *opcode;
-  opcode_hash_control = hash_new ();
+  opcode_hash_control = str_htab_create ();
 
   /* Insert names into hash table.  */
   for (count = 0, opcode = moxie_form1_opc_info; count++ < 64; opcode++)
-    hash_insert (opcode_hash_control, opcode->name, (char *) opcode);
+    str_hash_insert (opcode_hash_control, opcode->name, opcode, 0);
 
   for (count = 0, opcode = moxie_form2_opc_info; count++ < 4; opcode++)
-    hash_insert (opcode_hash_control, opcode->name, (char *) opcode);
+    str_hash_insert (opcode_hash_control, opcode->name, opcode, 0);
 
   for (count = 0, opcode = moxie_form3_opc_info; count++ < 10; opcode++)
-    hash_insert (opcode_hash_control, opcode->name, (char *) opcode);
-
-  target_big_endian = TARGET_BYTES_BIG_ENDIAN;
+    str_hash_insert (opcode_hash_control, opcode->name, opcode, 0);
 
   bfd_set_arch_mach (stdoutput, TARGET_ARCH, 0);
 }
@@ -165,13 +163,13 @@ md_assemble (char *str)
   int nlen = 0;
 
   /* Drop leading whitespace.  */
-  while (*str == ' ')
+  while (is_whitespace (*str))
     str++;
 
   /* Find the op code end.  */
   op_start = str;
   for (op_end = str;
-       *op_end && !is_end_of_line[*op_end & 0xff] && *op_end != ' ';
+       !is_end_of_stmt (*op_end) && !is_whitespace (*op_end);
        op_end++)
     nlen++;
 
@@ -180,7 +178,7 @@ md_assemble (char *str)
 
   if (nlen == 0)
     as_bad (_("can't find opcode "));
-  opcode = (moxie_opc_info_t *) hash_find (opcode_hash_control, op_start);
+  opcode = (moxie_opc_info_t *) str_hash_find (opcode_hash_control, op_start);
   *op_end = pend;
 
   if (opcode == NULL)
@@ -195,7 +193,7 @@ md_assemble (char *str)
     {
     case MOXIE_F2_A8V:
       iword = (1<<15) | (opcode->opcode << 12);
-      while (ISSPACE (*op_end))
+      while (is_whitespace (*op_end))
 	op_end++;
       {
 	expressionS arg;
@@ -216,7 +214,7 @@ md_assemble (char *str)
       break;
     case MOXIE_F1_AB:
       iword = opcode->opcode << 8;
-      while (ISSPACE (*op_end))
+      while (is_whitespace (*op_end))
 	op_end++;
       {
 	int dest, src;
@@ -226,7 +224,7 @@ md_assemble (char *str)
 	op_end++;
 	src  = parse_register_operand (&op_end);
 	iword += (dest << 4) + src;
-	while (ISSPACE (*op_end))
+	while (is_whitespace (*op_end))
 	  op_end++;
 	if (*op_end != 0)
 	  as_warn (_("extra stuff on line ignored"));
@@ -234,7 +232,7 @@ md_assemble (char *str)
       break;
     case MOXIE_F1_A4:
       iword = opcode->opcode << 8;
-      while (ISSPACE (*op_end))
+      while (is_whitespace (*op_end))
 	op_end++;
       {
 	expressionS arg;
@@ -242,7 +240,7 @@ md_assemble (char *str)
 	int regnum;
 
  	regnum = parse_register_operand (&op_end);
-	while (ISSPACE (*op_end))
+	while (is_whitespace (*op_end))
 	  op_end++;
 
 	iword += (regnum << 4);
@@ -268,7 +266,7 @@ md_assemble (char *str)
     case MOXIE_F1_M:
     case MOXIE_F1_4:
       iword = opcode->opcode << 8;
-      while (ISSPACE (*op_end))
+      while (is_whitespace (*op_end))
 	op_end++;
       {
 	expressionS arg;
@@ -286,19 +284,19 @@ md_assemble (char *str)
       break;
     case MOXIE_F1_NARG:
       iword = opcode->opcode << 8;
-      while (ISSPACE (*op_end))
+      while (is_whitespace (*op_end))
 	op_end++;
       if (*op_end != 0)
 	as_warn (_("extra stuff on line ignored"));
       break;
     case MOXIE_F1_A:
       iword = opcode->opcode << 8;
-      while (ISSPACE (*op_end))
+      while (is_whitespace (*op_end))
 	op_end++;
       {
 	int reg;
 	reg = parse_register_operand (&op_end);
-	while (ISSPACE (*op_end))
+	while (is_whitespace (*op_end))
 	  op_end++;
 	if (*op_end != 0)
 	  as_warn (_("extra stuff on line ignored"));
@@ -307,7 +305,7 @@ md_assemble (char *str)
       break;
     case MOXIE_F1_ABi:
       iword = opcode->opcode << 8;
-      while (ISSPACE (*op_end))
+      while (is_whitespace (*op_end))
 	op_end++;
       {
 	int a, b;
@@ -331,7 +329,7 @@ md_assemble (char *str)
 	  }
 	op_end++;
 	iword += (a << 4) + b;
-	while (ISSPACE (*op_end))
+	while (is_whitespace (*op_end))
 	  op_end++;
 	if (*op_end != 0)
 	  as_warn (_("extra stuff on line ignored"));
@@ -339,7 +337,7 @@ md_assemble (char *str)
       break;
     case MOXIE_F1_AiB:
       iword = opcode->opcode << 8;
-      while (ISSPACE (*op_end))
+      while (is_whitespace (*op_end))
 	op_end++;
       {
 	int a, b;
@@ -363,7 +361,7 @@ md_assemble (char *str)
 	op_end++;
 	b = parse_register_operand (&op_end);
 	iword += (a << 4) + b;
-	while (ISSPACE (*op_end))
+	while (is_whitespace (*op_end))
 	  op_end++;
 	if (*op_end != 0)
 	  as_warn (_("extra stuff on line ignored"));
@@ -371,7 +369,7 @@ md_assemble (char *str)
       break;
     case MOXIE_F1_4A:
       iword = opcode->opcode << 8;
-      while (ISSPACE (*op_end))
+      while (is_whitespace (*op_end))
 	op_end++;
       {
 	expressionS arg;
@@ -396,7 +394,7 @@ md_assemble (char *str)
 	op_end++;
 
  	a = parse_register_operand (&op_end);
-	while (ISSPACE (*op_end))
+	while (is_whitespace (*op_end))
 	  op_end++;
 	if (*op_end != 0)
 	  as_warn (_("extra stuff on line ignored"));
@@ -406,7 +404,7 @@ md_assemble (char *str)
       break;
     case MOXIE_F1_ABi2:
       iword = opcode->opcode << 8;
-      while (ISSPACE (*op_end))
+      while (is_whitespace (*op_end))
 	op_end++;
       {
 	expressionS arg;
@@ -414,7 +412,7 @@ md_assemble (char *str)
 	int a, b;
 
  	a = parse_register_operand (&op_end);
-	while (ISSPACE (*op_end))
+	while (is_whitespace (*op_end))
 	  op_end++;
 
 	if (*op_end != ',')
@@ -450,7 +448,7 @@ md_assemble (char *str)
 	  }
 	op_end++;
 
-	while (ISSPACE (*op_end))
+	while (is_whitespace (*op_end))
 	  op_end++;
 	if (*op_end != 0)
 	  as_warn (_("extra stuff on line ignored"));
@@ -460,7 +458,7 @@ md_assemble (char *str)
       break;
     case MOXIE_F1_AiB2:
       iword = opcode->opcode << 8;
-      while (ISSPACE (*op_end))
+      while (is_whitespace (*op_end))
 	op_end++;
       {
 	expressionS arg;
@@ -501,10 +499,7 @@ md_assemble (char *str)
 	op_end++;
 
  	b = parse_register_operand (&op_end);
-	while (ISSPACE (*op_end))
-	  op_end++;
-
-	while (ISSPACE (*op_end))
+	while (is_whitespace (*op_end))
 	  op_end++;
 	if (*op_end != 0)
 	  as_warn (_("extra stuff on line ignored"));
@@ -514,14 +509,14 @@ md_assemble (char *str)
       break;
     case MOXIE_F2_NARG:
       iword = opcode->opcode << 12;
-      while (ISSPACE (*op_end))
+      while (is_whitespace (*op_end))
 	op_end++;
       if (*op_end != 0)
 	as_warn (_("extra stuff on line ignored"));
       break;
     case MOXIE_F3_PCREL:
       iword = (3<<14) | (opcode->opcode << 10);
-      while (ISSPACE (*op_end))
+      while (is_whitespace (*op_end))
 	op_end++;
       {
 	expressionS arg;
@@ -531,13 +526,13 @@ md_assemble (char *str)
 		     (p - frag_now->fr_literal),
 		     2,
 		     &arg,
-		     TRUE,
+		     true,
 		     BFD_RELOC_MOXIE_10_PCREL);
       }
       break;
     case MOXIE_BAD:
       iword = 0;
-      while (ISSPACE (*op_end))
+      while (is_whitespace (*op_end))
 	op_end++;
       if (*op_end != 0)
 	as_warn (_("extra stuff on line ignored"));
@@ -549,7 +544,7 @@ md_assemble (char *str)
   md_number_to_chars (p, iword, 2);
   dwarf2_emit_insn (2);
 
-  while (ISSPACE (*op_end))
+  while (is_whitespace (*op_end))
     op_end++;
 
   if (*op_end != 0)
@@ -608,16 +603,16 @@ enum options
   OPTION_EL,
 };
 
-struct option md_longopts[] =
+const struct option md_longopts[] =
 {
   { "EB",          no_argument, NULL, OPTION_EB},
   { "EL",          no_argument, NULL, OPTION_EL},
   { NULL,          no_argument, NULL, 0}
 };
 
-size_t md_longopts_size = sizeof (md_longopts);
+const size_t md_longopts_size = sizeof (md_longopts);
 
-const char *md_shortopts = "";
+const char md_shortopts[] = "";
 
 int
 md_parse_option (int c ATTRIBUTE_UNUSED, const char *arg ATTRIBUTE_UNUSED)
@@ -779,8 +774,8 @@ tc_gen_reloc (asection *section ATTRIBUTE_UNUSED, fixS *fixP)
       return 0;
     }
 
-  relP = XNEW (arelent);
-  relP->sym_ptr_ptr = XNEW (asymbol *);
+  relP = notes_alloc (sizeof (arelent));
+  relP->sym_ptr_ptr = notes_alloc (sizeof (asymbol *));
   *relP->sym_ptr_ptr = symbol_get_bfdsym (fixP->fx_addsy);
   relP->address = fixP->fx_frag->fr_address + fixP->fx_where;
 
