@@ -1,5 +1,5 @@
 /* ns32k.c  -- Assemble on the National Semiconductor 32k series
-   Copyright (C) 1987-2024 Free Software Foundation, Inc.
+   Copyright (C) 1987-2025 Free Software Foundation, Inc.
 
    This file is part of GAS, the GNU Assembler.
 
@@ -1097,7 +1097,9 @@ parse (const char *line, int recursive_level)
   if (recursive_level <= 0)
     {
       /* Called from md_assemble.  */
-      for (lineptr = line; (*lineptr) != '\0' && (*lineptr) != ' '; lineptr++)
+      for (lineptr = line;
+	   (*lineptr) != '\0' && !is_whitespace (*lineptr);
+	   lineptr++)
 	continue;
 
       c = *lineptr;
@@ -2226,8 +2228,8 @@ tc_gen_reloc (asection *section ATTRIBUTE_UNUSED, fixS *fixp)
 
   code = reloc (fixp->fx_size, fixp->fx_pcrel, fix_im_disp (fixp));
 
-  rel = XNEW (arelent);
-  rel->sym_ptr_ptr = XNEW (asymbol *);
+  rel = notes_alloc (sizeof (arelent));
+  rel->sym_ptr_ptr = notes_alloc (sizeof (asymbol *));
   *rel->sym_ptr_ptr = symbol_get_bfdsym (fixp->fx_addsy);
   rel->address = fixp->fx_frag->fr_address + fixp->fx_where;
   if (fixp->fx_pcrel)
