@@ -1,5 +1,5 @@
 /* TI PRU assembler.
-   Copyright (C) 2014-2024 Free Software Foundation, Inc.
+   Copyright (C) 2014-2025 Free Software Foundation, Inc.
    Contributed by Dimitar Dimitrov <dimitar@dinux.eu>
    Based on tc-nios2.c
 
@@ -423,7 +423,7 @@ s_pru_set (int equiv)
      trying a directive.  This prevents
      us from polluting the name space.  */
   SKIP_WHITESPACE ();
-  if (is_end_of_line[(unsigned char) *input_line_pointer])
+  if (is_end_of_stmt (*input_line_pointer))
     {
       bool done = true;
       *endline = 0;
@@ -1441,7 +1441,7 @@ pru_parse_args (pru_insn_infoS *insn ATTRIBUTE_UNUSED, char *argstr,
       /* Strip trailing whitespace.  */
       len = strlen (parsed_args[i]);
       for (char *temp = parsed_args[i] + len - 1;
-	   len && ISSPACE (*temp);
+	   len && is_whitespace (*temp);
 	   temp--, len--)
 	*temp = '\0';
 
@@ -1766,10 +1766,11 @@ pru_fix_adjustable (fixS *fixp)
 arelent *
 tc_gen_reloc (asection *section ATTRIBUTE_UNUSED, fixS *fixp)
 {
-  arelent *reloc = XNEW (arelent);
-  reloc->sym_ptr_ptr = XNEW (asymbol *);
-  *reloc->sym_ptr_ptr = symbol_get_bfdsym (fixp->fx_addsy);
+  arelent *reloc;
 
+  reloc = notes_alloc (sizeof (arelent));
+  reloc->sym_ptr_ptr = notes_alloc (sizeof (asymbol *));
+  *reloc->sym_ptr_ptr = symbol_get_bfdsym (fixp->fx_addsy);
   reloc->address = fixp->fx_frag->fr_address + fixp->fx_where;
   reloc->addend = fixp->fx_offset;  /* fixp->fx_addnumber; */
 
@@ -1829,7 +1830,7 @@ pru_frob_label (symbolS *lab)
 static inline char *
 skip_space (char *s)
 {
-  while (*s == ' ' || *s == '\t')
+  while (is_whitespace (*s))
     ++s;
   return s;
 }
