@@ -1,5 +1,5 @@
 /* tc-rl78.c -- Assembler for the Renesas RL78
-   Copyright (C) 2011-2024 Free Software Foundation, Inc.
+   Copyright (C) 2011-2025 Free Software Foundation, Inc.
 
    This file is part of GAS, the GNU Assembler.
 
@@ -425,15 +425,13 @@ md_number_to_chars (char * buf, valueT val, int n)
 static void
 require_end_of_expr (const char *fname)
 {
-  while (* input_line_pointer == ' '
-	 || * input_line_pointer == '\t')
+  while (is_whitespace (* input_line_pointer))
     input_line_pointer ++;
 
-  if (! * input_line_pointer
-      || strchr ("\n\r,", * input_line_pointer)
+  if (is_end_of_stmt (* input_line_pointer)
+      || * input_line_pointer == ','
       || strchr (comment_chars, * input_line_pointer)
-      || strchr (line_comment_chars, * input_line_pointer)
-      || strchr (line_separator_chars, * input_line_pointer))
+      || strchr (line_comment_chars, * input_line_pointer))
     return;
 
   as_bad (_("%%%s() must be outermost term in expression"), fname);
@@ -1257,11 +1255,11 @@ tc_gen_reloc (asection * seg ATTRIBUTE_UNUSED, fixS * fixp)
       fixp->fx_subsy = NULL;
     }
 
-  reloc[0]		  = XNEW (arelent);
-  reloc[0]->sym_ptr_ptr   = XNEW (asymbol *);
-  * reloc[0]->sym_ptr_ptr = symbol_get_bfdsym (fixp->fx_addsy);
-  reloc[0]->address       = fixp->fx_frag->fr_address + fixp->fx_where;
-  reloc[0]->addend        = fixp->fx_offset;
+  reloc[0] = notes_alloc (sizeof (arelent));
+  reloc[0]->sym_ptr_ptr = notes_alloc (sizeof (asymbol *));
+  *reloc[0]->sym_ptr_ptr = symbol_get_bfdsym (fixp->fx_addsy);
+  reloc[0]->address = fixp->fx_frag->fr_address + fixp->fx_where;
+  reloc[0]->addend = fixp->fx_offset;
 
   if (fixp->fx_r_type == BFD_RELOC_RL78_32_OP
       && fixp->fx_subsy)
@@ -1269,13 +1267,13 @@ tc_gen_reloc (asection * seg ATTRIBUTE_UNUSED, fixS * fixp)
       fixp->fx_r_type = BFD_RELOC_RL78_DIFF;
     }
 
-#define OPX(REL,SYM,ADD)							\
-  reloc[rp]		   = XNEW (arelent);		\
-  reloc[rp]->sym_ptr_ptr   = XNEW (asymbol *);		\
-  reloc[rp]->howto         = bfd_reloc_type_lookup (stdoutput, REL);		\
-  reloc[rp]->addend        = ADD;						\
-  * reloc[rp]->sym_ptr_ptr = SYM;						\
-  reloc[rp]->address       = fixp->fx_frag->fr_address + fixp->fx_where;	\
+#define OPX(REL,SYM,ADD)						\
+  reloc[rp] = notes_alloc (sizeof (arelent));				\
+  reloc[rp]->sym_ptr_ptr = notes_alloc (sizeof (asymbol *));		\
+  reloc[rp]->howto = bfd_reloc_type_lookup (stdoutput, REL);		\
+  reloc[rp]->addend = ADD;						\
+  *reloc[rp]->sym_ptr_ptr = SYM;					\
+  reloc[rp]->address = fixp->fx_frag->fr_address + fixp->fx_where;	\
   reloc[++rp] = NULL
 #define OPSYM(SYM) OPX(BFD_RELOC_RL78_SYM, SYM, 0)
 
