@@ -1,5 +1,5 @@
 /* tc-c30.c -- Assembly code for the Texas Instruments TMS320C30
-   Copyright (C) 1998-2024 Free Software Foundation, Inc.
+   Copyright (C) 1998-2025 Free Software Foundation, Inc.
    Contributed by Steven Haworth (steve@pm.cse.rmit.edu.au)
 
    This file is part of GAS, the GNU Assembler.
@@ -180,7 +180,7 @@ md_begin (void)
 	if (ISALPHA (c) || c == '_' || c == '.' || ISDIGIT (c))
 	  identifier_chars[c] = c;
 
-	if (c == ' ' || c == '\t')
+	if (is_whitespace (c))
 	  space_chars[c] = c;
 
 	if (c == '_')
@@ -287,7 +287,7 @@ tic30_find_parallel_insn (char *current_line, char *next_line)
   char *parallel_insn;
 
   debug ("In tic30_find_parallel_insn()\n");
-  while (!is_end_of_line[(unsigned char) *next_line])
+  while (!is_end_of_stmt (*next_line))
     {
       if (*next_line == PARALLEL_SEPARATOR
 	  && *(next_line + 1) == PARALLEL_SEPARATOR)
@@ -326,7 +326,7 @@ tic30_find_parallel_insn (char *current_line, char *next_line)
 	  int char_ptr = 0;
 	  char c;
 
-	  while (!is_end_of_line[(unsigned char) (c = *line)])
+	  while (!is_end_of_stmt (c = *line))
 	    {
 	      if (is_opcode_char (c) && search_status == NONE)
 		{
@@ -1206,7 +1206,7 @@ md_atof (int what_statement_type,
   debug ("literal = %s\n", literalP);
   debug ("line = ");
   token = input_line_pointer;
-  while (!is_end_of_line[(unsigned char) *input_line_pointer]
+  while (!is_end_of_stmt (*input_line_pointer)
 	 && (*input_line_pointer != ','))
     {
       debug ("%c", *input_line_pointer);
@@ -1367,9 +1367,8 @@ tc_gen_reloc (asection *section ATTRIBUTE_UNUSED, fixS *fixP)
 #undef MAP
 #undef F
 
-  rel = XNEW (arelent);
-  gas_assert (rel != 0);
-  rel->sym_ptr_ptr = XNEW (asymbol *);
+  rel = notes_alloc (sizeof (arelent));
+  rel->sym_ptr_ptr = notes_alloc (sizeof (asymbol *));
   *rel->sym_ptr_ptr = symbol_get_bfdsym (fixP->fx_addsy);
   rel->address = fixP->fx_frag->fr_address + fixP->fx_where;
   rel->addend = 0;
