@@ -1,5 +1,5 @@
 /* listing.c - maintain assembly listings
-   Copyright (C) 1991-2024 Free Software Foundation, Inc.
+   Copyright (C) 1991-2025 Free Software Foundation, Inc.
 
    This file is part of GAS, the GNU Assembler.
 
@@ -220,17 +220,6 @@ static FILE *list_file;
 
 static char *data_buffer;
 
-/* Prototypes.  */
-static void listing_message (const char *, const char *);
-static file_info_type *file_info (const char *);
-static void new_frag (void);
-static void listing_page (list_info_type *);
-static unsigned int calc_hex (list_info_type *);
-static void print_lines (list_info_type *, unsigned int, const char *,
-			 unsigned int);
-static void list_symbol_table (void);
-static void listing_listing (char *);
-
 static void
 listing_message (const char *name, const char *message)
 {
@@ -362,8 +351,7 @@ listing_newline (char *ps)
 	  int seen_slash = 0;
 
 	  for (copy = input_line_pointer;
-	       *copy && (seen_quote
-			 || is_end_of_line [(unsigned char) *copy] != 1);
+	       seen_quote ? *copy : !is_end_of_line (*copy);
 	       copy++)
 	    {
 	      if (seen_slash)
@@ -443,6 +431,51 @@ listing_newline (char *ps)
 #endif
 }
 
+/* Set listing context back to where it was when input was parsed, to allow
+   associating late code/data emission to segments with their origin.  */
+
+struct list_info_struct *listing_override_tail (struct list_info_struct *info)
+{
+  struct list_info_struct *prev = listing_tail;
+  const fragS *frag;
+
+  if (!info)
+    return NULL;
+
+  listing_tail = info;
+
+  /* The first frag created by listing_newline() is still associated with the
+     earlier line.  For the adjustment done below this property doesn't hold,
+     though.  */
+  frag = info->frag;
+  if (frag->line != info)
+    frag = frag->fr_next;
+
+  /* Check whether there's any other output data already for this line.  Replace
+     info->frag only if there's none.  This is to cover for contributions to
+     multiple sections from a single line not being properly represented in the
+     listing, at the time of writing.  Prefer the listing to show any "ordinary"
+     code/data contributions over any .eh_frame ones.  (This way multiple .cfi_*
+     on a single line will also have all their contributions listed, rather
+     than just those from the last such directive.)  */
+  for (; frag; frag = frag->fr_next)
+    if (frag->line != info
+	|| (frag->fr_type != rs_dummy
+	    && (frag->fr_type != rs_fill
+		|| frag->fr_fix
+		|| (frag->fr_var && frag->fr_offset))))
+      break;
+
+  if (!frag || frag->line != info)
+    {
+      new_frag ();
+      info->frag = frag_now;
+      new_frag ();
+    }
+
+  return prev;
+}
+
 /* Attach all current frags to the previous line instead of the
    current line.  This is called by the MIPS backend when it discovers
    that it needs to add some NOP instructions; the added NOP
@@ -793,7 +826,7 @@ calc_hex (list_info_type *list)
 	  data_buffer_size += 2;
 	  octet_in_frag++;
 	}
-      if (frag_ptr->fr_type == rs_fill)
+      if (frag_ptr->fr_type == rs_fill || frag_ptr->fr_type == rs_fill_nop)
 	{
 	  unsigned int var_rep_max = octet_in_frag;
 	  unsigned int var_rep_idx = octet_in_frag;
@@ -818,28 +851,6 @@ calc_hex (list_info_type *list)
 		var_rep_idx = var_rep_max;
 	    }
 	}
-      else if (frag_ptr->fr_type == rs_fill_nop && frag_ptr->fr_opcode)
-	{
-	  gas_assert (!octet_in_frag);
-
-	  /* Print as many bytes from fr_opcode as is sensible.  */
-	  while (octet_in_frag < (unsigned int) frag_ptr->fr_offset
-		 && data_buffer_size < MAX_BYTES - 3)
-	    {
-	      if (address == ~(unsigned int) 0)
-		address = frag_ptr->fr_address / OCTETS_PER_BYTE;
-
-	      sprintf (data_buffer + data_buffer_size,
-		       "%02X",
-		       frag_ptr->fr_opcode[octet_in_frag] & 0xff);
-	      data_buffer_size += 2;
-
-	      octet_in_frag++;
-	    }
-
-	  free (frag_ptr->fr_opcode);
-	  frag_ptr->fr_opcode = NULL;
-	}
 
       frag_ptr = frag_ptr->fr_next;
     }
@@ -1152,7 +1163,7 @@ debugging_pseudo (list_info_type *list ATTRIBUTE_UNUSED, const char *line)
   in_debug = false;
 #endif
 
-  while (ISSPACE (*line))
+  while (is_whitespace (*line))
     line++;
 
   if (*line != '.')
@@ -1581,7 +1592,7 @@ listing_title (int depth)
     {
       if (quoted
 	  ? *input_line_pointer == '\"'
-	  : is_end_of_line[(unsigned char) *input_line_pointer])
+	  : is_end_of_stmt (*input_line_pointer))
 	{
 	  if (listing)
 	    {
@@ -1626,58 +1637,4 @@ listing_source_file (const char *file)
     listing_tail->hll_file = file_info (file);
 }
 
-#else
-
-/* Dummy functions for when compiled without listing enabled.  */
-
-void
-listing_list (int on)
-{
-  s_ignore (0);
-}
-
-void
-listing_eject (int ignore)
-{
-  s_ignore (0);
-}
-
-void
-listing_psize (int ignore)
-{
-  s_ignore (0);
-}
-
-void
-listing_nopage (int ignore)
-{
-  s_ignore (0);
-}
-
-void
-listing_title (int depth)
-{
-  s_ignore (0);
-}
-
-void
-listing_file (const char *name)
-{
-}
-
-void
-listing_newline (char *name)
-{
-}
-
-void
-listing_source_line (unsigned int n)
-{
-}
-
-void
-listing_source_file (const char *n)
-{
-}
-
 #endif
