@@ -110,12 +110,6 @@ int symbol_table_frozen;
 
 symbolS *abs_section_sym;
 
-/* Remember the value of dot when parsing expressions.  */
-addressT dot_value;
-
-/* The frag that dot_value is based from.  */
-fragS *dot_frag;
-
 /* Relocs generated by ".reloc" pseudo.  */
 struct reloc_list* reloc_list;
 
@@ -162,8 +156,8 @@ fix_new_internal (fragS *frag,		/* Which frag?  */
   fixP->fx_addsy = add_symbol;
   fixP->fx_subsy = sub_symbol;
   fixP->fx_offset = offset;
-  fixP->fx_dot_value = dot_value;
-  fixP->fx_dot_frag = dot_frag;
+  fixP->fx_dot_frag = symbol_get_frag_and_value (&dot_symbol,
+						 &fixP->fx_dot_value);
   fixP->fx_pcrel = pcrel;
   fixP->fx_r_type = r_type;
   fixP->fx_pcrel_adjust = 0;
@@ -423,7 +417,7 @@ cvt_frag_to_fill (segT sec ATTRIBUTE_UNUSED, fragS *fragP)
     case rs_org:
     case rs_space:
 #ifdef HANDLE_ALIGN
-      HANDLE_ALIGN (fragP);
+      HANDLE_ALIGN (sec, fragP);
 #endif
     skip_align:
       know (fragP->fr_next != NULL);
@@ -743,11 +737,11 @@ resolve_reloc_expr_symbols (void)
 	  else if (sym != NULL && sec != NULL)
 	    {
 	      /* Convert relocs against local symbols to refer to the
-	         corresponding section symbol plus offset instead.  Keep
-	         PC-relative relocs of the REL variety intact though to
+		 corresponding section symbol plus offset instead.  Keep
+		 PC-relative relocs of the REL variety intact though to
 		 prevent the offset from overflowing the relocated field,
-	         unless it has enough bits to cover the whole address
-	         space.  */
+		 unless it has enough bits to cover the whole address
+		 space.  */
 	      if (S_IS_LOCAL (sym)
 		  && S_IS_DEFINED (sym)
 		  && !symbol_section_p (sym)
@@ -1454,17 +1448,17 @@ compress_frag (bool use_zstd, void *ctx, const char *contents, int in_size,
   while (in_size > 0)
     {
       /* Reserve all the space available in the current chunk.
-         If none is available, start a new frag.  */
+	 If none is available, start a new frag.  */
       avail_out = obstack_room (ob);
       if (avail_out <= 0)
-        {
-          obstack_finish (ob);
-          f = frag_alloc (ob);
+	{
+	  obstack_finish (ob);
+	  f = frag_alloc (ob, 0);
 	  f->fr_type = rs_fill;
-          (*last_newf)->fr_next = f;
-          *last_newf = f;
-          avail_out = obstack_room (ob);
-        }
+	  (*last_newf)->fr_next = f;
+	  *last_newf = f;
+	  avail_out = obstack_room (ob);
+	}
       if (avail_out <= 0)
 	as_fatal (_("can't extend frag"));
       next_out = obstack_next_free (ob);
@@ -1516,16 +1510,8 @@ compress_debug (bfd *abfd, asection *sec, void *xxx ATTRIBUTE_UNUSED)
 
   /* Create a new frag to contain the compression header.  */
   struct obstack *ob = &seginfo->frchainP->frch_obstack;
-  fragS *first_newf = frag_alloc (ob);
-  if (obstack_room (ob) < header_size)
-    first_newf = frag_alloc (ob);
-  if (obstack_room (ob) < header_size)
-    as_fatal (ngettext ("can't extend frag %lu char",
-			"can't extend frag %lu chars",
-			(unsigned long) header_size),
-	      (unsigned long) header_size);
+  fragS *first_newf = frag_alloc (ob, header_size);
   fragS *last_newf = first_newf;
-  obstack_blank_fast (ob, header_size);
   last_newf->fr_type = rs_fill;
   last_newf->fr_fix = header_size;
   char *header = last_newf->fr_literal;
@@ -1583,7 +1569,7 @@ compress_debug (bfd *abfd, asection *sec, void *xxx ATTRIBUTE_UNUSED)
 	  fragS *newf;
 
 	  obstack_finish (ob);
-	  newf = frag_alloc (ob);
+	  newf = frag_alloc (ob, 0);
 	  newf->fr_type = rs_fill;
 	  last_newf->fr_next = newf;
 	  last_newf = newf;
@@ -1667,6 +1653,19 @@ write_contents (bfd *abfd ATTRIBUTE_UNUSED,
       offsetT count;
 
       gas_assert (f->fr_type == rs_fill || f->fr_type == rs_fill_nop);
+
+      count = f->fr_offset;
+      fill_literal = f->fr_literal + f->fr_fix;
+      if (f->fr_type == rs_fill_nop && count > 0)
+	{
+	  md_generate_nops (f, fill_literal, count, *fill_literal);
+	  /* md_generate_nops updates fr_fix and fr_var.  */
+	  f->fr_offset = (f->fr_next->fr_address - f->fr_address
+			  - f->fr_fix) / f->fr_var;
+	  count = f->fr_offset;
+	  fill_literal = f->fr_literal + f->fr_fix;
+	}
+
       if (f->fr_fix)
 	{
 	  x = bfd_set_section_contents (stdoutput, sec,
@@ -1685,39 +1684,6 @@ write_contents (bfd *abfd ATTRIBUTE_UNUSED,
 	}
 
       fill_size = f->fr_var;
-      count = f->fr_offset;
-      fill_literal = f->fr_literal + f->fr_fix;
-
-      if (f->fr_type == rs_fill_nop)
-	{
-	  gas_assert (count >= 0 && fill_size == 1);
-	  if (count > 0)
-	    {
-	      char *buf = xmalloc (count);
-	      md_generate_nops (f, buf, count, *fill_literal);
-	      x = bfd_set_section_contents
-		(stdoutput, sec, buf, (file_ptr) offset,
-		 (bfd_size_type) count);
-	      if (!x)
-		as_fatal (ngettext ("can't fill %ld byte "
-				    "in section %s of %s: '%s'",
-				    "can't fill %ld bytes "
-				    "in section %s of %s: '%s'",
-				    (long) count),
-			  (long) count,
-			  bfd_section_name (sec),
-			  bfd_get_filename (stdoutput),
-			  bfd_errmsg (bfd_get_error ()));
-	      offset += count;
-#ifndef NO_LISTING
-	      if (listing & LISTING_LISTING)
-		f->fr_opcode = buf;
-	      else
-#endif
-		free (buf);
-	    }
-	  continue;
-	}
 
       gas_assert (count >= 0);
       if (fill_size && count)
@@ -1818,9 +1784,8 @@ set_symtab (void)
   if (nsyms)
     {
       int i;
-      bfd_size_type amt = (bfd_size_type) nsyms * sizeof (asymbol *);
 
-      asympp = (asymbol **) bfd_alloc (stdoutput, amt);
+      asympp = notes_alloc (nsyms * sizeof (asymbol *));
       symp = symbol_rootP;
       for (i = 0; i < nsyms; symp = symbol_next (symp))
 	if (!symbol_removed_p (symp)
@@ -1933,27 +1898,27 @@ subsegs_finish (void)
 }
 
 #ifdef OBJ_ELF
+
 static void
 create_obj_attrs_section (void)
 {
-  segT s;
-  char *p;
-  offsetT size;
-  const char *name;
-
-  size = bfd_elf_obj_attr_size (stdoutput);
+  offsetT size = bfd_elf_obj_attr_size (stdoutput);
   if (size == 0)
     return;
 
-  name = get_elf_backend_data (stdoutput)->obj_attrs_section;
-  if (!name)
+  const char *name = get_elf_backend_data (stdoutput)->obj_attrs_section;
+  if (name == NULL)
+    /* Note: .gnu.attributes is different from GNU_BUILD_ATTRS_SECTION_NAME
+       (a.k.a .gnu.build.attributes). The first one seems to be used by some
+       backends like PowerPC to store the build attributes. The second one is
+       used to store build notes.  */
     name = ".gnu.attributes";
-  s = subseg_new (name, 0);
+  segT s = subseg_new (name, 0);
   elf_section_type (s)
     = get_elf_backend_data (stdoutput)->obj_attrs_section_type;
   bfd_set_section_flags (s, SEC_READONLY | SEC_DATA);
   frag_now_fix ();
-  p = frag_more (size);
+  char *p = frag_more (size);
   bfd_elf_set_obj_attr_contents (stdoutput, (bfd_byte *)p, size);
 
   subsegs_finish_section (s);
@@ -3000,9 +2965,7 @@ relax_segment (struct frag *segment_frag_root, segT segment, int pass)
 		      struct obstack *ob = &seginfo->frchainP->frch_obstack;
 		      struct frag *newf;
 
-		      newf = frag_alloc (ob);
-		      obstack_blank_fast (ob, fragP->fr_var);
-		      obstack_finish (ob);
+		      newf = frag_alloc (ob, fragP->fr_var);
 		      memcpy (newf, fragP, SIZEOF_STRUCT_FRAG);
 		      memcpy (newf->fr_literal,
 			      fragP->fr_literal + fragP->fr_fix,
